#!/bin/bash

# Configuration
PAPER_URL=https://papermc.io/api/v2/projects/paper

# Enter server directory
cd /papermc

# Determine minecraft version (MC_VERSION) and build number (PAPER_BUILD). If they are set to
# "latest", use PaperMC's API to determine which version and build number is the latest.
if [ ${MC_VERSION} = latest ]
  then
    # Get the latest MC version
    MC_VERSION=$(curl ${PAPER_URL} | jq -r '.versions[-1]') # "-r" is needed because the output has quotes otherwise
fi

PAPER_URL=${PAPER_URL}/versions/${MC_VERSION}
if [ ${PAPER_BUILD} = latest ]
  then
    # Get the latest build
    PAPER_BUILD=$(curl ${PAPER_URL} | jq '.builds[-1]')
fi

# Generate the name of the server JAR file we are about to download from
# the minecraft version and the paper build number we have determined above.
JAR_NAME=paper-${MC_VERSION}-${PAPER_BUILD}.jar

# If the jar is not yet downloaded, download it
if [ ! -e ${JAR_NAME} ]
  then
    # Cleanup older, left-over JAR files
    rm -f *.jar
    # Download the file
    curl ${PAPER_URL}/builds/${PAPER_BUILD}/downloads/${JAR_NAME} --output ${JAR_NAME}
    # Accept EULA
    if [ ! -e eula.txt ]
      then
        java -jar ${JAR_NAME}
        sed -i 's/false/true/g' eula.txt
    fi
fi

# Download base plugins, worlds and configuration by Tobias Müller
if [ ! -d "plugins" ]
 then
   curl -u ${TM_BASE_AUTH} ${TM_BASE_URL} --output initial.tar.gz
   tar -xvzf initial.tar.gz
fi

exec_stop () {
  echo "\n## Recieved SIGINT/SIGTERM; Shutting down container..."
  screen -wipe
  screen -r mc -p 0 -X stuff "stop^M" #sendet einen string an die session und mit ^M wird ein 'Enter' simuliert
  sleep 30 #Soll dafür sorgen dass der Container nicht sofort das script beendet währen in der session noch der befehl ausgeführt wird.
  exit 0
}

while true
  do
    screen -S mc -X select . ; SCRUN=$? #Queryabfrage um rauszufindne ob screensession noch aktiv ist. Return: bool
    if [ ${SCRUN} = 0 ]
      then # Server läuft, zeige letzte zeilen vom log an...
        clear # Aber erstmal sauber machen ^^
        tail /papermc/logs/latest.log
        trap exec_stop TERM INT # SIGTERM ^C etc abfangen und funktion für gracefull shutdown aufrufen
        sleep 10 # CPU Zeit sparen da eine downtime von max 10s + Startzeit des javapplets vertretbar ist kann bei bedarf beliebig erhöht werden.
      else # Server läuft nicht, abgestürzt, gestoppt über minecraft /stop 
        screen -dmS mc java -server -Xms${MC_RAM} -Xmx${MC_RAM} ${JAVA_OPTS} -jar ${JAR_NAME} nogui #Startupbefehl inkl. ENVs vom Dockerfile
        screen -wipe # sollte der server mal abstürzen oder sont irgednwie ein fehler auftreten wird damit zombiesessions entfernt.
    fi
  done
